#include <video.h>
#include <kernel/registros.h>
#include <kernel/puertos.h>
#include <kernel/definiciones.h>

/*
 * Modulo que implementa la logica de video. 
 */

extern int iFnSysImprimirCaracter (int, char);

int iFnCompararCadenas (const char *, const char *);

/**
 * iFnImprimirCadena: Imprime una cadena por pantalla.
 * @param cnstCadena cadena a imprimir.
 * @return cantidad de caracteres impresos
 */
int
iFnImprimirCadena (int hVentana, const char *cnstCadena)
{
  while (*cnstCadena != 0)
    {
      iFnSysImprimirCaracter (hVentana, *cnstCadena);
      cnstCadena++;
    }
  return 0;
}

/**
 * iFnLongitudCadena: Devuelve la longitud de una cadena (teniendo en cuenta
 *                  que el caracter de finalizacion de una cadena sera el 
 *                  caracter 0, ascii 48).
 * @param cnstCadena cadena a la cual se le contara la longitud
 * @return la longitud de la cadena
 */
int
iFnLongitudCadena (const char *cnstCadena)
{
  static int staiLongitud;
  staiLongitud = 0;
  while (cnstCadena[staiLongitud++] != '\0');

  return staiLongitud;
}

/**
 * vFnImprimirEntero: Imprime un entero o un doble word (32 bits) por pantalla
 * @param nro entero a imprimir.
 */
void
vFnImprimirEntero (int hVentana, const int cniNumero)
{
  // 2^32 = 4294967296 --> a lo sumo el entero tendra 10 digitos + signo + \0
  static char stastCadena[12];

  vFnItoa (cniNumero, (char *) stastCadena);
  iFnImprimirCadena (hVentana, stastCadena);

}

/**
 * vFnImprimirWord: Imprime un word (16 bits) por pantalla
 * @param nro entero a imprimir.
 */
void
vFnImprimirWord (int hVentana, const word cnwNumero)
{
  // 2^16 = 65535 --> a lo sumo el entero tendra 5 digitos + signo + \0
  static char stastCadena[7];

  vFnWtoa (cnwNumero, (char *) stastCadena);
  iFnImprimirCadena (hVentana, stastCadena);

}

/**
 * vFnImprimirByte: Imprime un byte (8 bits) por pantalla.
 * @param nro entero a imprimir.
 */
void
vFnImprimirByte (int hVentana, const byte cnbyNumero)
{
  // 2^16 = 65535 --> a lo sumo el entero tendra 5 digitos + signo + \0
  static char stastCadena[7];

  vFnBtoa (cnbyNumero, (char *) stastCadena);
  iFnImprimirCadena (hVentana, stastCadena);
}

/**
 * vFnImprimirHexa: Imprime un entero o doble word (32 bits) en base 16 por 
 *                pantalla.
 * @param nro entero a imprimir.
 */
void
vFnImprimirHexa (int hVentana, const int cniNumero)
{
  // 2^32 = 0xFFFFFFFF --> a lo sumo el entero tendra 8 digitos + signo + \0
  static char stastCadena[10];

  vFnItoh (cniNumero, (char *) stastCadena);
  iFnImprimirCadena (hVentana, stastCadena);

}

/**
 * vFnImprimirWordHexa: Imprime un word (16 bits) en base 16 por pantalla.
 * @param nro entero a imprimir.
 */
void
vFnImprimirWordHexa (int hVentana, const word cnwNumero)
{
  // 2^16 = 0xFFFF --> a lo sumo el entero tendra 4 digitos + signo + \0
  static char stastCadena[6];

  vFnWtoh (cnwNumero, (char *) stastCadena);
  iFnImprimirCadena (hVentana, stastCadena);
}

/**
 * vFnImprimirByteHexa: Imprime un byte (8 bits) en base 16 por pantalla.
 * @param nro entero a imprimir.
 */
void
vFnImprimirByteHexa (int hVentana, const byte cnbyNumero)
{
  // 2^16 = 0xFF --> a lo sumo el entero tendra 2 digitos + signo + \0
  static char stastCadena[4];

  vFnBtoh (cnbyNumero, (char *) stastCadena);
  iFnImprimirCadena (hVentana, stastCadena);
}

/**
 * vFnImprimir: Imprime una cadena por pantalla, recibiendo argumentos variables,
 *           en forma analoga a la funcion de biblioteca de C printf().
 * @param stastCadena cadena a imprimir y parsear para conocer los argumentos variables
 * @param ... de cero a N argumentos con parametros para la funcion imprimir().
 *
 * basada en la funcion printf del GNU/gcc
 * @link http://www.gnu.org
 */
void
vFnImprimirVentana (int hVentana, const char *cnstCadena, ...)
{
  int iFlags;
  int bInterrupcionesHabilitadas = 0;	// indica si las interrupciones estaban
  // habilitadas al entrar a la funcion

  va_list (lista_argumentos);
  va_start (lista_argumentos, cnstCadena);

__asm__ ("pushf\n pop %%eax": "=a" (iFlags):);

  // si estaban habilitadas, aqui se deshabilitan
  if (iFlags & 0x200)
    {
      __asm__ ("cli"::);
      bInterrupcionesHabilitadas = 1;
    }


  while (*cnstCadena)
    {
      if (*cnstCadena == '%')
	{
	  cnstCadena++;
	  switch (*cnstCadena)
	    {
	    case 'c':
	      iFnSysImprimirCaracter (hVentana,
				      va_arg (lista_argumentos, char));
	      break;
	    case 'd':
	      vFnImprimirEntero (hVentana, va_arg (lista_argumentos, int));
	      break;
	    case 'w':
	      vFnImprimirEntero (hVentana, va_arg (lista_argumentos, word));
	      break;
	    case 'b':
	      vFnImprimirByte (hVentana, va_arg (lista_argumentos, byte));
	      break;
	    case 'x':
	      cnstCadena++;
	      if (*cnstCadena == 'w')
		{
		  vFnImprimirWordHexa (hVentana,
				       va_arg (lista_argumentos, word));
		}
	      else if (*cnstCadena == 'b')
		{
		  vFnImprimirByteHexa (hVentana,
				       va_arg (lista_argumentos, byte));
		}
	      else
		{
		  cnstCadena--;
		  vFnImprimirHexa (hVentana, va_arg (lista_argumentos, int));
		}
	      break;
	    case 's':
	      iFnImprimirCadena (hVentana, va_arg (lista_argumentos, char *));
	      break;
	    }
	}
      else
	{
	  iFnSysImprimirCaracter (hVentana, *cnstCadena);
	}
      cnstCadena++;
    }
  va_end (lista_argumentos);

  if (bInterrupcionesHabilitadas)
    __asm__ ("sti"::);
}

void
vFnImprimir (const char *cnstCadena, ...)
{
  int iFlags;
  int bInterrupcionesHabilitadas = 0;	// indica si las interrupciones estaban
  // habilitadas al entrar a la funcion

  va_list (lista_argumentos);
  va_start (lista_argumentos, cnstCadena);

__asm__ ("pushf\n pop %%eax": "=a" (iFlags):);

  // si estaban habilitadas, aqui se deshabilitan
  if (iFlags & 0x200)
    {
      __asm__ ("cli"::);
      bInterrupcionesHabilitadas = 1;
    }

  while (*cnstCadena)
    {
      if (*cnstCadena == '%')
	{
	  cnstCadena++;
	  switch (*cnstCadena)
	    {
	    case 'c':
	      iFnSysImprimirCaracter (0, va_arg (lista_argumentos, char));
	      break;
	    case 'd':
	      vFnImprimirEntero (0, va_arg (lista_argumentos, int));
	      break;
	    case 'w':
	      vFnImprimirEntero (0, va_arg (lista_argumentos, word));
	      break;
	    case 'b':
	      vFnImprimirByte (0, va_arg (lista_argumentos, byte));
	      break;
	    case 'x':
	      cnstCadena++;
	      if (*cnstCadena == 'w')
		{
		  vFnImprimirWordHexa (0, va_arg (lista_argumentos, word));
		}
	      else if (*cnstCadena == 'b')
		{
		  vFnImprimirByteHexa (0, va_arg (lista_argumentos, byte));
		}
	      else
		{
		  cnstCadena--;
		  vFnImprimirHexa (0, va_arg (lista_argumentos, int));
		}
	      break;
	    case 's':
	      iFnImprimirCadena (0, va_arg (lista_argumentos, char *));
	      break;
	    }
	}
      else
	{
	  iFnSysImprimirCaracter (0, *cnstCadena);
	}
      cnstCadena++;
    }
  va_end (lista_argumentos);

  if (bInterrupcionesHabilitadas)
    __asm__ ("sti"::);
}

/**
 * vFnItoa: transforma un entero en ascii
 * @param number numero a transformar
 * @param stBuffer posicion de memoria donde se almacenara el numero
 *
 * <pre>
 * pseudocodigo:
 *    el numero se va dividiendo por diez y el resto mas 48 es lo
 *    que hay que imprimir.  Ej:
 *
 * int nro = 345
 *
 *    resto = nro%10; // resto = 5
 *    nro /= 10; // nro = 34
 *    etc....
 * 
 * debido a que el nro se imprimiria al revez, se almacena en un aux.
 * </pre>
 */
void
vFnItoa (int iNumero, char *stBuffer)
{
  static int staiAux;
  staiAux = iNumero;

  if (iNumero < 0)
    {
      iNumero *= -1;
      *stBuffer++ = '-';
    }

  do
    {
      *stBuffer++ = 48;
    }
  while ((staiAux /= 10) != 0);

  *stBuffer-- = '\0';
  while (iNumero != 0)
    {
      staiAux = iNumero % 10;
      iNumero = iNumero / 10;
      *stBuffer-- = staiAux + 48;
    }
}

/**
 * Tres funciones agregadas por Luis Moyano
 * tal vez le sirvan a alguien
 */
/**
 *Fecha 20-11-2004
 * dwFnCtod: transforma una cadena en dword (sin signo
 * @param stBuffer posicion de memoria donde viene el numero
 * @return el numero transformado
 */

dword
dwFnCtod (char *stBuffer)
{

  int iN, iX, iLongitud;
  dword dwPonderacion = 1;
  dword dwNumero = 0;

  iLongitud = iFnLongitudCadena (stBuffer);
  for (iN = iLongitud - 2; iN >= 0; iN--)
    {
      if (stBuffer[iN] < 48 || stBuffer[iN] > 57)
	{
	  return NULL;
	}
      dwPonderacion = 1;
      for (iX = 0; iX < (iLongitud - iN - 2); iX++)
	{
	  dwPonderacion *= 10;
	}
      dwNumero += (((dword) (stBuffer[iN]) - 48) * dwPonderacion);
    }
  return dwNumero;
}

/**
 *Fecha 20-11-2004
 * wFnCtow: transforma una cadena en word (sin signo
 * @param stBuffer posicion de memoria donde viene el numero
 * @return el numero transformado
 */
word
wFnCtow (char *stBuffer)
{

  int iN, iX, iLongitud;
  word wPonderacion = 1;
  word wNumero = 0;

  iLongitud = iFnLongitudCadena (stBuffer);
  for (iN = iLongitud - 2; iN >= 0; iN--)
    {
      if (stBuffer[iN] < 48 || stBuffer[iN] > 57)
	{
	  return NULL;
	}
      wPonderacion = 1;
      for (iX = 0; iX < (iLongitud - iN - 2); iX++)
	{
	  wPonderacion *= 10;
	}
      wNumero += (((word) (stBuffer[iN]) - 48) * wPonderacion);
    }
  return wNumero;
}

/**
 *Fecha 20-11-2004
 * byFnCtob: transforma una cadena en byte (sin signo)
 * @param stBuffer posicion de memoria donde viene el numero
 * @return el numero transformado
 */
byte
byFnCtob (char *stBuffer)
{

  int iN, iX, iLongitud;
  byte byPonderacion = 1;
  byte byNumero = 0;

  iLongitud = iFnLongitudCadena (stBuffer);
  for (iN = iLongitud - 2; iN >= 0; iN--)
    {
      if (stBuffer[iN] < 48 || stBuffer[iN] > 57)
	{
	  return NULL;
	}
      byPonderacion = 1;
      for (iX = 0; iX < (iLongitud - iN - 2); iX++)
	{
	  byPonderacion *= 10;
	}
      byNumero += (((byte) (stBuffer[iN]) - 48) * byPonderacion);
    }
  return byNumero;

}

/**
 * iFnCtoi: transforma una cadena en entero
 * @param stBuffer posicion de memoria donde viene el numero
 * @return el numero transformado
 */
/*int iFnCtoi ( char* stBuffer ) {
   int iN, iX, iLongitud;
   int iPonderacion = 1;
   int iNumero = 0;

   iLongitud = iFnLongitudCadena (stBuffer);
   for (iN = iLongitud - 2; iN >= 0; iN--) {
      if ( stBuffer[iN] < 48 || stBuffer[iN] > 57 ) {
         return NULL;
      }
      iPonderacion = 1;
      for (iX = 0; iX < (iLongitud - iN - 2); iX++) {
         iPonderacion *= 10;
      }
      iNumero += ( ((int)(stBuffer[iN]) - 48) * iPonderacion );
   }
   return iNumero;
}*/
/**
 *Modificacion hecha por Luis Moyano
 *Esta version al contrario de la anterior
 *verifica si el parametro es un numero negativo 
 *y si es asi lo devuelve como tal.
 *Fecha 20-11-2004
 * iFnCtoi: transforma una cadena en entero
 * @param stBuffer posicion de memoria donde viene el numero
 * @return el numero transformado
 */
int
iFnCtoi (char *stBuffer)
{

  int iN, iX, iLongitud;
  int iPonderacion = 1;
  int iNumero = 0;

  iLongitud = iFnLongitudCadena (stBuffer);
  for (iN = iLongitud - 2; iN >= 0; iN--)
    {
      if (stBuffer[iN] < 48 || stBuffer[iN] > 57)
	{
	  if (stBuffer[0] == 45)
	    {
	      //si es un negativo
	      iNumero = iNumero * (-1);
	      return iNumero;
	    }
	  else
	    return NULL;
	}
      iPonderacion = 1;
      for (iX = 0; iX < (iLongitud - iN - 2); iX++)
	{
	  iPonderacion *= 10;
	}
      iNumero += (((int) (stBuffer[iN]) - 48) * iPonderacion);
    }
  return iNumero;
}

/**
 * vFnItoh: transforma un entero en ascii ( formato hexa )
 * @param number numero a transformar
 * @param stBuffer posicion de memoria donde se almacenara el numero
 */
void
vFnItoh (int iNumero, char *stBuffer)
{
  static int staiAux;
  staiAux = iNumero;

  if (iNumero < 0)
    {
      iNumero *= -1;
      *stBuffer++ = '-';
    }

  do
    {
      *stBuffer++ = 48;
    }
  while ((staiAux /= 16) != 0);

  *stBuffer-- = '\0';
  while (iNumero != 0)
    {
      staiAux = iNumero % 16;
      iNumero = iNumero / 16;
      if (staiAux < 10)
	*stBuffer-- = staiAux + 48;
      else
	*stBuffer-- = staiAux + 55;
    }
}

/**
 * vFnWtoh: transforma un word en ascii (formato hexa)
 * @param number numero a transformar
 * @param stBuffer posicion de memoria donde se almacenara el numero
 */
void
vFnWtoh (word wNumero, char *stBuffer)
{
  static word stawNumero;
  stawNumero = wNumero;

  do
    {
      *stBuffer++ = 48;
    }
  while ((stawNumero /= 16) != 0);

  *stBuffer-- = '\0';
  while (wNumero != 0)
    {
      stawNumero = wNumero % 16;
      wNumero = wNumero / 16;
      if (stawNumero < 10)
	*stBuffer-- = stawNumero + 48;
      else
	*stBuffer-- = stawNumero + 55;
    }
}

/**
 * vFnBtoh: transforma un byte en ascii ( formato hexa )
 * @param number numero a transformar
 * @param stBuffer posicion de memoria donde se almacenara el numero
 */
void
vFnBtoh (byte byNumero, char *stBuffer)
{
  static byte stabyAux;
  stabyAux = byNumero;

  do
    {
      *stBuffer++ = 48;
    }
  while ((stabyAux /= 16) != 0);

  *stBuffer-- = '\0';
  while (byNumero != 0)
    {
      stabyAux = byNumero % 16;
      byNumero = byNumero / 16;
      if (stabyAux < 10)
	*stBuffer-- = stabyAux + 48;
      else
	*stBuffer-- = stabyAux + 55;
    }
}

/**
 * vFnWtoa: transforma un word en ascii
 * @param number numero a transformar
 * @param stBuffer posicion de memoria donde se almacenara el numero
 */
void
vFnWtoa (word wNumero, char *stBuffer)
{
  static word stawAux;
  stawAux = wNumero;

  do
    {
      *stBuffer++ = 48;
    }
  while ((stawAux /= 10) != 0);

  *stBuffer-- = '\0';
  while (wNumero != 0)
    {
      stawAux = wNumero % 10;
      wNumero = wNumero / 10;
      *stBuffer-- = stawAux + 48;
    }
}

/**
 * vFnBtoa: transforma un byte en ascii
 * @param number numero a transformar
 * @param stBuffer posicion de memoria donde se almacenara el numero
 */
void
vFnBtoa (byte byNumero, char *stBuffer)
{
  static byte stabyAux;
  stabyAux = byNumero;

  do
    {
      *stBuffer++ = 48;
    }
  while ((stabyAux /= 10) != 0);

  *stBuffer-- = '\0';
  while (byNumero != 0)
    {
      stabyAux = byNumero % 10;
      byNumero = byNumero / 10;
      *stBuffer-- = stabyAux + 48;
    }
}



/******************************************************************************
Funcin: iFnHtoi
Descripcin:  Recibe una cadena de carateres en un formato hexa predeterminado
              y devuelve el equivalente en decimal.

Recibe:   *Puntero a la cadena a interpretar, (terminada en '/0' !!!!!)
Devuelve: *Integer con el valor decimal luego de la conversin.

Fecha ltima modificacin: 09/04/2006
*******************************************************************************/
int
iFnHtoi (char *c)
{
  int iN, iJ = 0;
  for (; (*c != '\0');)
    {
      iN = *c++ - 48;
      9 < iN ? iN -= 7 : iN;
      iJ <<= 4;
      iJ |= iN & 15;
    }
  //vFnImprimir ("\nEl Valor ingresado en Hexa es igual a %d en decimal", iJ);
  return iJ;
}


/******************************************************************************
Funcin: iFnXtoi
Descripcin:  Recibe una cadena de carateres en un formato hexa o entero
              y devuelve el equivalente en decimal. Para ello interpreta la
              naturaleza de la cadena y llama a iFnXtoi si es hexa o a iFnCtoi si
              es entero. En el caso que sea hexa, normaliza el string para
              que termine en /0 y que no contenga el "0x" inicial o el "h"
              final.

Recibe:   *Puntero a la cadena a interpretar, (terminada en '/0' !!!!!)
Devuelve: *Integer con el valor decimal luego de la conversin.

Fecha ltima modificacin: 09/04/2006
*******************************************************************************/

int
iFnXtoi (char *c)
{
  int tipo = 0;
  char *p = c;
  while (*c != '\0')
    {
      if (*c == 'x' || *c == 'h')
	{
	  if (*c == 'x')
	    {
	      p += 2;
	    }
	  if (*c == 'h')
	    {
	      *c = '\0';
	    }
	  tipo = 1;
	}
      c++;
    }

  if (tipo == 0)
    {
      return (iFnCtoi (p));
    }
  else
    {
      return (iFnHtoi (p));
    }
}




/**
 * iFnCompararCadenas: compara dos cadenas
 * @param cnstCadena1 primer cadena a comparar
 * @param cnstCadena2 segunda cadena a comparar
 * @return -1 si son diferentes, 0 si son iguales
 */
int
iFnCompararCadenas (const char *cnstCadena1, const char *cnstCadena2)
{
  int iN;

  if (iFnLongitudCadena (cnstCadena1) != iFnLongitudCadena (cnstCadena2))
    {
      return -1;
    }

  iN = 0;
  while (cnstCadena1[iN] != '\0')
    {
      if (cnstCadena1[iN] != cnstCadena2[iN++])
	{
	  return -1;
	}
    }
  return 0;
}



/**********************************************************************************
Funcion:imprimirClockTicks

Ultima Modificacin: 21/09/2004

***********************************************************************************/
void
vFnImprimirContextSwitch (int iColor, int iPid, char *stNombre,
			  int iGDTIndice)
{

  vFnImprimirVentana (1, "\nCSW: PID %d\tTAREA=%s\tINDICE GDT: %d", iPid,
		      stNombre, iGDTIndice);
}

/**********************************************************************************
Funcion:vFnImprimirPrompt

Ultima Modificacin: 21/09/2004

***********************************************************************************/

void
vFnImprimirPrompt ()
{
  vFnImprimir ("%s", STRPROMPTLINE);
}



/**********************************************************************************
Funcion:vFnImprimirOk

Ultima Modificacin: 21/09/2004

***********************************************************************************/

void
vFnImprimirOk (int iLinea)
{
  int iFlags;
  unsigned short int bInterrupcionesHabilitadas = 0;
__asm__ ("pushf\n pop %%eax": "=a" (iFlags):);

  // si estaban habilitadas, aqui se deshabilitan
  if (iFlags & 0x200)
    {
      __asm__ ("cli"::);
      bInterrupcionesHabilitadas = 1;
    }

  //sys_setear_x(linea);
  vFnSysSetearColor (HWND_COMANDO, VERDE_CLARO);
  vFnImprimir ("		[ HECHO ] ");
  vFnSysSetearColor (HWND_COMANDO, BLANCO);

  if (bInterrupcionesHabilitadas)
    __asm__ ("sti"::);

}
void
vFnImprimirNOk (int iLinea)
{
  int iFlags;
  unsigned short int bInterrupcionesHabilitadas = 0;
__asm__ ("pushf\n pop %%eax": "=a" (iFlags):);

  // si estaban habilitadas, aqui se deshabilitan
  if (iFlags & 0x200)
    {
      __asm__ ("cli"::);
      bInterrupcionesHabilitadas = 1;
    }

  //sys_setear_x(linea);
  vFnSysSetearColor (HWND_COMANDO, ROJO);
  vFnImprimir ("		[ NOK!!!] ");
  vFnSysSetearColor (HWND_COMANDO, BLANCO);

  if (bInterrupcionesHabilitadas)
    __asm__ ("sti"::);

}

